import re
import logging

from colorama import init as colorama_init
from colorama import Style
from colorama import Fore
from colorama import Back
from colorama import Cursor

logger = logging.getLogger(__name__)

MAX_CHARS = 120
FILL = 2


class StatusLine(object):
    """ Status Line implementation
    """

    def __init__(self, index, fill=None, regex=None):
        """ class constructor
        """
        logger.debug('executing StatusLine constructor')
        colorama_init()
        self.fill = StatusLine._get_fill(fill)
        if not regex:
            regex = {}
        self.text = ''
        self.regex = regex
        self.index = index
        self.duration = None

    def __str__(self):
        """ return string interpretation of class instance
        """
        index_fill = self.fill['index']
        bright_yellow = Style.BRIGHT + Fore.YELLOW + Back.BLACK
        index = f"{bright_yellow}{str(self.index).zfill(index_fill)}{Style.RESET_ALL}"
        text = self.text
        if self.duration:
            text = f'{text.strip()} - {self.duration}'
        return f"{index}: {text}"

    def reset(self):
        """ reset status line
        """
        pass

    def match(self, text):
        """ call match functions and return on first success
        """
        regex = self.regex.get('text')
        if regex:
            match = re.match(regex, text)
            if match:
                self.text = StatusLine._sanitize(text)
        else:
            self.text = StatusLine._sanitize(text)

    @staticmethod
    def _sanitize(text):
        """ sanitize text
        """
        if text:
            text = text.splitlines()[0]
            if len(text) > MAX_CHARS:
                text = f'{text[0:MAX_CHARS - 3]}...'
            else:
                text = text.ljust(MAX_CHARS)
        return text

    @staticmethod
    def _get_fill(data):
        """ return fill dictionary derived from data values
        """
        fill = {}
        if not data:
            fill['index'] = FILL
        else:
            fill['index'] = len(str(data.get('max_index', FILL * '-')))
        return fill



#   -*- coding: utf-8 -*-
import sys
import cursor
import logging
from colorama import init as colorama_init
from colorama import Cursor
from progress1bar import ProgressBar
from mp4ansi.statusline import StatusLine

logger = logging.getLogger(__name__)

MAX_LINES = 75
HIDE_CURSOR = '\x1b[?25l'
SHOW_CURSOR = '\x1b[?25h'
CLEAR_EOL = '\033[K'


class Terminal():
    """ simple ANSI terminal
        represented internally as a list of dictionaries
        ability to move up or down and print text at specified index
    """
    def __init__(self, number_of_lines, config=None, durations=None, create=True):
        """ class constructor
        """
        logger.debug('executing Terminal constructor')
        self.validate_lines(number_of_lines)
        if not config:
            config = {}
        self.validate_config(config)
        self.config = config
        colorama_init()
        self.current = 0
        self.durations = durations
        if create:
            self.terminal = self.create(number_of_lines)

    def validate_lines(self, number_of_lines):
        """ validate number_of_lines
        """
        if number_of_lines < 0 or number_of_lines > MAX_LINES:
            raise ValueError(f'lines must be between 1-{MAX_LINES}')

    def validate_config(self, config):
        """ validate progress bar config
        """
        if 'progress_bar' in config:
            keys = config['progress_bar'].keys()
            required_keys = ['total', 'count_regex']

            if not all(item in keys for item in required_keys):
                raise ValueError(f'progress_bar config does not contain all the required parameters: {required_keys}')

            if not isinstance(config['progress_bar']['total'], (str, int)):
                raise ValueError('progress_bar.total must be an integer or string')

    def create_progress_bars(self, lines):
        """ create and return list of progress bars
        """
        regex = {}
        regex['count'] = self.config['progress_bar']['count_regex']
        regex['alias'] = self.config.get('id_regex')
        total = None
        if isinstance(self.config['progress_bar']['total'], str):
            regex['total'] = self.config['progress_bar']['total']
        else:
            total = self.config['progress_bar']['total']
        fill = {}
        fill['max_total'] = self.config['progress_bar'].get('max_total')
        fill['max_index'] = lines
        fill['max_completed'] = self.config['progress_bar'].get('max_completed')
        terminal = []
        message = self.config['progress_bar'].get('progress_message')
        for index in range(lines):
            progress_bar = ProgressBar(index, total=total, fill=fill, regex=regex, completed_message=message, aware=False)
            terminal.append(progress_bar)
        return terminal

    def create_status_lines(self, lines):
        """ create return list of status lines
        """
        regex = {}
        regex['text'] = self.config.get('text_regex')
        fill = {}
        fill['max_index'] = lines
        terminal = []
        for index in range(lines):
            status_line = StatusLine(index, fill=fill, regex=regex)
            terminal.append(status_line)
        return terminal

    def create(self, number_of_lines):
        """ return list of dictionaries representing terminal
        """
        logger.debug('creating terminal')
        if 'progress_bar' in self.config:
            return self.create_progress_bars(number_of_lines)
        return self.create_status_lines(number_of_lines)

    def write(self, index, force=False):
        """ move to index and print terminal line at index
        """
        if sys.stderr.isatty() or force:
            move_char = self.get_move_char(index)
            print(f'{move_char}{CLEAR_EOL}', end='', file=sys.stderr)
            print(self.terminal[index], file=sys.stderr)
            sys.stderr.flush()
            self.current += 1

    def write_line(self, index, text):
        """ write terminal line at index
        """
        self.terminal[index].match(text)
        self.write(index)

    def write_lines(self, add_duration=False, force=False):
        """ write lines to terminal
        """
        logger.debug('writing terminal')
        for index, _ in enumerate(self.terminal):
            if add_duration:
                self.terminal[index].duration = self.durations.get(str(index))
            self.write(index, force=force)

    def reset(self, index):
        """ reset termnal index
        """
        logger.debug(f'resetting terminal at index {index}')
        self.terminal[index].reset()

    def get_move_char(self, index):
        """ return char to move to index
        """
        move_char = ''
        if index < self.current:
            move_char = self.move_up(index)
        elif index > self.current:
            move_char = self.move_down(index)
        return move_char

    def move_down(self, index):
        """ return char to move down to index and update current
        """
        diff = index - self.current
        self.current += diff
        return Cursor.DOWN(diff)

    def move_up(self, index):
        """ return char to move up to index and update current
        """
        diff = self.current - index
        self.current -= diff
        return Cursor.UP(diff)

    def show_cursor(self):
        """ show cursor
        """
        if sys.stderr.isatty():
            cursor.show()

    def hide_cursor(self):
        """ hide cursor
        """
        if sys.stderr.isatty():
            cursor.hide()



#!/usr/bin/env python
#   -*- coding: utf-8 -*-

from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def pre_install_script(self):
        pass

    def post_install_script(self):
        pass

    def run(self):
        self.pre_install_script()

        _install.run(self)

        self.post_install_script()

if __name__ == '__main__':
    setup(
        name = 'mp4ansi',
        version = '0.3.6',
        description = 'A simple ANSI-based terminal emulator that provides multi-processing capabilities.',
        long_description = '# mp4ansi #\n[![GitHub Workflow Status](https://github.com/soda480/mp4ansi/workflows/build/badge.svg)](https://github.com/soda480/mp4ansi/actions)\n[![Code Coverage](https://codecov.io/gh/soda480/mp4ansi/branch/main/graph/badge.svg?token=6NTX6LSP7Q)](https://codecov.io/gh/soda480/mp4ansi)\n[![Code Grade](https://www.code-inspector.com/project/20694/status/svg)](https://frontend.code-inspector.com/project/20694/dashboard)\n[![vulnerabilities](https://img.shields.io/badge/vulnerabilities-None-green)](https://pypi.org/project/bandit/)\n[![PyPI version](https://badge.fury.io/py/mp4ansi.svg)](https://badge.fury.io/py/mp4ansi)\n[![python](https://img.shields.io/badge/python-3.6-teal)](https://www.python.org/downloads/)\n\n\nA simple ANSI-based terminal that provides various capabilities for showing off and/or scaling out your programs execution. MP4ansi is an abstraction of multiprocessing that leverages both the Terminal and ProgressBar. See the [examples](https://github.com/soda480/mp4ansi/tree/master/examples) for more detail.\n\n## Installation ##\n```bash\npip install mp4ansi\n```\n\n## Examples ##\n\nVarious [examples](https://github.com/soda480/mp4ansi/tree/master/examples) are included to demonstrate the mp4ansi package. To run the examples, build the Docker image and run the Docker container using the instructions described in the [Development](#development) section.\n\n### `MP4ansi`\n\nMP4ansi will scale execution of a specified function across multiple background processes, where each process is mapped to specific line on the terminal. As the function executes its log messages will automatically be written to the respective line on the terminal. The number of processes along with the arguments to provide each process is specified as a list of dictionaries. The number of elements in the list will dictate the total number of processes to execute (as well as the number of lines in the terminal). The result of each function is written to the respective dictionary element and can be interogated upon completion. \n\nMP4ansi is a subclass of `mpmq`, see the [mpmq](https://pypi.org/project/mpmq/) for more information.\n\nHere is a simple example:\n\n```python\nfrom mp4ansi import MP4ansi\nimport random, names, logging\nlogger = logging.getLogger(__name__)\n\ndef do_work(*args):\n    total = random.randint(50, 100)\n    logger.debug(f\'processing total of {total}\')\n    for _ in range(total):\n        logger.debug(f\'processed {names.get_full_name()}\')\n    return total\n\ndef main():\n    process_data = [{} for item in range(8)]\n    print(\'Procesing names...\')\n    MP4ansi(function=do_work, process_data=process_data).execute()\n    print(f"Total names processed {sum([item[\'result\'] for item in process_data])}")\n\nif __name__ == \'__main__\':\n    main()\n```\n\nExecuting the code above ([example1](https://github.com/soda480/mp4ansi/tree/master/examples/example1.py)) results in the following:\n![example](https://raw.githubusercontent.com/soda480/mp4ansi/master/docs/images/example1.gif)\n\n**Note** the function being executed `do_work` has no context about multiprocessing or the terminal; it simply perform a function on a given dataset. MP4ansi takes care of setting up the multiprocessing, setting up the terminal, and maintaining the thread-safe queues that are required for inter-process communication.\n\nLet\'s update the example to add a custom identifer for each process and to show execution as a progress bar. To do this we need to provide additonal configuration via the optional `config` parameter. Configuration is supplied as a dictionary; `id_regex` instructs how to query for the identifer from the log messages. For the progress bar, we need to specify `total` and `count_regex` to instruct how to query for the total and for when to count that an item has been processed. The value for these settings are specified as regular expressions and will match the function log messages, thus we need to ensure our function has log statements for these. If each instance of your function executes on a static data range then you can specify total as an `int`, but in this example the data range is dynamic, i.e. each process will execute on varying data ranges.\n\n```python\nfrom mp4ansi import MP4ansi\nimport names, random, logging\nlogger = logging.getLogger(__name__)\n\ndef do_work(*args):\n    logger.debug(f\'processor is {names.get_last_name()}\')\n    total = random.randint(50, 125)\n    logger.debug(f\'processing total of {total}\')\n    for _ in range(total):\n        logger.debug(f\'processed {names.get_full_name()}\')\n    return total\n\ndef main():\n    process_data = [{} for item in range(8)]\n    config = {\n        \'id_regex\': r\'^processor is (?P<value>.*)$\',\n        \'progress_bar\': {\n            \'total\': r\'^processing total of (?P<value>\\d+)$\',\n            \'count_regex\': r\'^processed (?P<value>.*)$\',\n            \'progress_message\': \'Finished processing names\'}}\n    print(\'Procesing names...\')\n    MP4ansi(function=do_work, process_data=process_data, config=config).execute()\n    print(f"Total names processed {sum([item[\'result\'] for item in process_data])}")\n\nif __name__ == \'__main__\':\n    main()\n```\n\nExecuting the code above ([example2](https://github.com/soda480/mp4ansi/tree/master/examples/example2.py)) results in the following:\n![example](https://raw.githubusercontent.com/soda480/mp4ansi/master/docs/images/example2.gif)\n\n### `Terminal`\n\nThe package also exposes a `Terminal` class if you wish to consume the terminal capabilities without executing background processes. Here is an example for how to do that:\n```python\nfrom mp4ansi import Terminal\nfrom essential_generators import DocumentGenerator\nimport time, random\n\ndef main():\n    print(\'generating random sentences...\')\n    count = 15\n    docgen = DocumentGenerator()\n    terminal = Terminal(count)\n    terminal.write_lines()\n    terminal.hide_cursor()\n    for _ in range(800):\n        index = random.randint(0, count - 1)\n        terminal.write_line(index, docgen.sentence())\n        time.sleep(.01)\n    terminal.write_lines(force=True)\n    terminal.show_cursor()\n\nif __name__ == \'__main__\':\n    main()\n```\n\nExecuting the code above ([example5](https://github.com/soda480/mp4ansi/tree/master/examples/example5.py)) results in the following:\n![example](https://raw.githubusercontent.com/soda480/mp4ansi/master/docs/images/example5.gif)\n\n## Development ##\n\nClone the repository and ensure the latest version of Docker is installed on your development server.\n\nBuild the Docker image:\n```sh\ndocker image build \\\n-t \\\nmp4ansi:latest .\n```\n\nRun the Docker container:\n```sh\ndocker container run \\\n--rm \\\n-it \\\n-v $PWD:/mp4ansi \\\nmp4ansi:latest \\\n/bin/sh\n```\n\nExecute the build:\n```sh\npyb -X\n```\n',
        long_description_content_type = 'text/markdown',
        classifiers = [
            'Development Status :: 4 - Beta',
            'Environment :: Console',
            'Environment :: Other Environment',
            'Intended Audience :: Developers',
            'Intended Audience :: System Administrators',
            'License :: OSI Approved :: Apache Software License',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3.6',
            'Topic :: Software Development :: Libraries',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: System :: Networking',
            'Topic :: System :: Systems Administration'
        ],
        keywords = '',

        author = 'Emilio Reyes',
        author_email = 'soda480@gmail.com',
        maintainer = '',
        maintainer_email = '',

        license = 'Apache License, Version 2.0',

        url = 'https://github.com/soda480/mp4ansi',
        project_urls = {},

        scripts = [],
        packages = ['mp4ansi'],
        namespace_packages = [],
        py_modules = [],
        entry_points = {},
        data_files = [],
        package_data = {},
        install_requires = [
            'mpmq',
            'cursor',
            'colorama',
            'progress1bar'
        ],
        dependency_links = [],
        zip_safe = True,
        cmdclass = {'install': install},
        python_requires = '',
        obsoletes = [],
    )
